@@ -3,14 +3,14 @@
  * fe-auth.c
  *	   The front-end (client) authorization routines
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * NOTE: the error message strings returned by this module must not
  * exceed INITIAL_EXPBUFFER_SIZE (currently 256 bytes).
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/interfaces/libpq/fe-auth.c,v 1.63 2001/11/05 17:46:37 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/interfaces/libpq/fe-auth.c,v 1.71.2.1 2005/12/13 03:36:14 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -63,10 +63,14 @@
  * common definitions for generic fe/be routines
  */
 
+#define STARTUP_MSG		7		/* Initialise a connection */
+#define STARTUP_KRB4_MSG	10	/* krb4 session follows */
+#define STARTUP_KRB5_MSG	11	/* krb5 session follows */
+#define STARTUP_PASSWORD_MSG	14		/* Password follows */
+
 struct authsvc
 {
-	char		name[NAMEDATALEN];		/* service nickname (for command
-										 * line) */
+	const char *name;			/* service nickname (for command line) */
 	MsgType		msgtype;		/* startup packet header type */
 	int			allowed;		/* initially allowed (before command line
 								 * option parsing)? */
@@ -138,7 +142,7 @@ pg_krb4_init()
 	{
 		char		tktbuf[MAXPGPATH];
 
-		(void) sprintf(tktbuf, "%s@%s", tkt_string(), realm);
+		(void) snprintf(tktbuf, sizeof(tktbuf), "%s@%s", tkt_string(), realm);
 		krb_set_tkt_string(tktbuf);
 	}
 }
@@ -403,9 +407,18 @@ pg_krb5_sendauth(char *PQerrormsg, int sock,
 	{
 		if (retval == KRB5_SENDAUTH_REJECTED && err_ret)
 		{
+#if defined(HAVE_KRB5_ERROR_TEXT_DATA)
 			snprintf(PQerrormsg, PQERRORMSG_LENGTH,
 			  libpq_gettext("Kerberos 5 authentication rejected: %*s\n"),
 					 err_ret->text.length, err_ret->text.data);
+#elif defined(HAVE_KRB5_ERROR_E_DATA)
+			snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+			  libpq_gettext("Kerberos 5 authentication rejected: %*s\n"),
+					 err_ret->e_data->length,
+					 (const char *) err_ret->e_data->data);
+#else
+#error "bogus configuration"
+#endif
 		}
 		else
 		{
@@ -448,13 +461,6 @@ pg_local_sendauth(char *PQerrormsg, PGconn *conn)
 	/* Point to start of first structure */
 	struct cmsghdr *cmsg = (struct cmsghdr *) cmsgmem;
 #endif
-#ifdef HAVE_STRUCT_SOCKCRED
-	/* Prevent padding */
-	char		cmsgmem[sizeof(struct cmsghdr) + sizeof(struct sockcred)];
-
-	/* Point to start of first structure */
-	struct cmsghdr *cmsg = (struct cmsghdr *) cmsgmem;
-#endif
 
 	/*
 	 * The backend doesn't care what we send here, but it wants exactly
@@ -605,13 +611,13 @@ fe_sendauth(AuthRequest areq, PGconn *conn, const char *hostname,
 		case AUTH_REQ_PASSWORD:
 			if (password == NULL || *password == '\0')
 			{
-				(void) sprintf(PQerrormsg,
-							   "fe_sendauth: no password supplied\n");
+				(void) snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+								"fe_sendauth: no password supplied\n");
 				return STATUS_ERROR;
 			}
 			if (pg_password_sendauth(conn, password, areq) != STATUS_OK)
 			{
-				(void) sprintf(PQerrormsg,
+				(void) snprintf(PQerrormsg, PQERRORMSG_LENGTH,
 				 "fe_sendauth: error sending password authentication\n");
 				return STATUS_ERROR;
 			}
@@ -683,7 +689,7 @@ fe_getauthsvc(char *PQerrormsg)
 char *
 fe_getauthname(char *PQerrormsg)
 {
-	char	   *name = (char *) NULL;
+	const char *name = (char *) NULL;
 	char	   *authn = (char *) NULL;
 	MsgType		authsvc;
 
